@@ -115,11 +115,11 @@ class TestAvailability(unittest.TestCase):
                  (os.path.exists("/sys/devices/system/cpu/cpufreq") or
                   os.path.exists("/sys/devices/system/cpu/cpu0/cpufreq")))
         self.assertEqual(hasattr(psutil, "cpu_freq"),
-                         linux or MACOS or WINDOWS)
+                         linux or MACOS or WINDOWS or FREEBSD)
 
     def test_sensors_temperatures(self):
         self.assertEqual(
-            hasattr(psutil, "sensors_temperatures"), LINUX)
+            hasattr(psutil, "sensors_temperatures"), LINUX or FREEBSD)
 
     def test_sensors_fans(self):
         self.assertEqual(hasattr(psutil, "sensors_fans"), LINUX)
@@ -337,7 +337,7 @@ class TestFetchAllProcesses(unittest.TestCase):
                             self.assertEqual(err.name, p.name())
                         assert str(err)
                         assert err.msg
-                    except Exception as err:
+                    except Exception:
                         s = '\n' + '=' * 70 + '\n'
                         s += "FAIL: test_%s (proc=%s" % (name, p)
                         if ret != default:
